/*ML: hdr_patch*/

#include "frame.h"
#include "povproto.h"
#include "hdr.h"
#include "povray.h"
#include "file_pov.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


void Read_HDR_Image(IMAGE *Image, char *filename)
{
  POV_ISTREAM *f;
  int height,width,cline;
  char line[BUFSIZ];
  int  i, j;
  int  code, val;
  unsigned char original[2048][4];


  if ((f = Locate_File(filename, POV_File_Image_HDR, NULL,true)) == NULL)
  {
    Error("Error opening HDR image.\n");
    return;	
  }

  {
    char s[BUFSIZ];

    /*while ((g = fgets(s, sizeof(s), f)) &&
	   (strlen(g) > 1));*/
    while (f->getline(s, BUFSIZ) && (strlen(s) > 1));

  }

  // Image size
  f->getline(line, BUFSIZ); 
  if ((sscanf(line, "+X %d +Y %d\n", &height, &width) != 2) &&
      (sscanf(line, "-Y %d +X %d\n", &height, &width) != 2)) 
    Error ("Bad HDR header");

  Image->iwidth = width;
  Image->iheight = height;
  Image->width = (DBL)Image->iwidth;
  Image->height = (DBL)Image->iheight;
  Image->Colour_Map_Size = 0;
  Image->Colour_Map = NULL;

  Image->data.rgb_float_lines = (IMAGE_FLOAT_LINE *)POV_MALLOC(Image->iheight * sizeof(IMAGE_FLOAT_LINE), "HDR image");


  for (cline=0;cline<height;cline++)
  {
    Image->data.rgb_float_lines[cline].red = (float *)POV_MALLOC(width*sizeof(float), "HDR image line");
    Image->data.rgb_float_lines[cline].green = (float *)POV_MALLOC(width*sizeof(float), "HDR image line");
    Image->data.rgb_float_lines[cline].blue = (float *)POV_MALLOC(width*sizeof(float), "HDR image line");

    // determine scanline type
    if ((i = f->Read_Byte()) == EOF)
      Error("Error reading HDR file");
    if (i != 2)
      Error("Error reading HDR file");

    f->Read_Byte();     // ignore
    j = f->Read_Byte();
    i = f->Read_Byte();
    if (((j << 8) | i) != width)
      Error("Error reading HDR file");;		// length mismatch! 
    
    // read each component 
    for (i = 0; i < 4; i++)
      for (j = 0; j < width; ) 
      {
        if ((code = f->Read_Byte()) == EOF)
	        Error("Error reading HDR file");
        if (code > 128) 
        {
          // run 
	        code &= 127;
	        val = f->Read_Byte();
	        while (code--)
	          original[j++][i] = val;
        } 
        else			
          // non-run 
	        while (code--)
	          original[j++][i] = f->Read_Byte();
      }
    for (j = 0; j < width; j++) 
    {
      int r, g, b, e;
      double v;

      r = original[j][0];
      g = original[j][1];
      b = original[j][2];
      e = original[j][3];

      v = ldexp(1.0, (int)(e - (128+8)));
      Image->data.rgb_float_lines[cline].red[j] = (float)((double)(r + 0.5) * v);
      Image->data.rgb_float_lines[cline].green[j] = (float)((double)(g + 0.5) * v);
      Image->data.rgb_float_lines[cline].blue[j] = (float)((double)(b + 0.5) * v);
    }
  }

  POV_DELETE(f, POV_ISTREAM);
}

/*-ML: hdr_patch*/